import socket
import string
import sys

def print_bytes(octets):
    for b in octets:
        print('{:02x} '.format(b), end = '')
    print()

def prepare_data(size):
    nb = 1000
    data = ""
    for i in range(int(size/nb)):
        data = data + string.ascii_uppercase[i]*nb
    rest = size - int(size/nb)*nb
    data = data + string.ascii_uppercase[int(size/nb)]*rest

    return data

def build_ipv6_header(src, dest, size):
    # first 32 bits 
    #   version (4 bits), traffic class (8 bits), flow label (20 bits)
    #   version has to be 6, choose the rest two fields 0
    ver = 6 << 28 # 32 - 4 = 28
    tc  = 0 << 20 # 32 - 4 - 8 = 20
    fl  = 0
    row1 = (ver | tc | fl).to_bytes(4, byteorder='big')
    #print_bytes(row1)

    # second 32 bits
    #   payload length (16 bits), next header (8 bits), hop limits (8 bits)
    le = size << 16    # 32 - 16 = 16
    nh = 0x90 << 8 # 32 - 16 - 8 = 8
    hl = 64
    row2 = (le | nh | hl).to_bytes(4, byteorder='big')
    #print_bytes(row2)
    
    row3 = socket.inet_pton(socket.AF_INET6, src)
    #print_bytes(row3)

    row4 = socket.inet_pton(socket.AF_INET6, dest)
    #print_bytes(row4)

    ipv6hdr = b"".join([row1, row2, row3, row4])
    print_bytes(ipv6hdr)

    return ipv6hdr;
    

def transmit(src, dest, size):
    data = prepare_data(size)
    sock = socket.socket(family=socket.AF_INET6, type=socket.SOCK_RAW\
        , proto=socket.IPPROTO_IPV6)

    ipv6hdr = build_ipv6_header(src, dest, size)
    
    nb = sock.sendto(b"".join([ipv6hdr, data.encode()]), (dest, 0))
    print("Sent an IPv6 packet of " + str(nb) + " bytes")
    
    sock.close()
    
def usage(prog):
    print("Usage: " + prog + " source desintation payload_length")

def main(argv):
    if len(sys.argv) < 4:
        usage(argv[0])
        sys.exit(1)
    transmit(argv[1], argv[2], int(argv[3]))

if __name__ == "__main__":
    main(sys.argv)
